<?php

/*! CFDI 3.3 2017
 * © 2017 Realhost SA de CV
 */

/**
 * @summary     ERP
 * @description CFDI 3.3 Functions
 * @version     1.2
 * @file        CFDI33.php
 * @author      Ulises Carreón Alvarez
 * @contact     ulises.carreon@realhost.com.mx
 * @copyright   Copyright 2008-2020 Realhost SA de CV.
 **/

namespace App\Core;


include "XML.php";
include "Comprobante.php";
include "Concepto.php";
include "ConceptoImpuesto.php";
include "Conceptos.php";
include "Domicilio.php";
include "Emisor.php";
include "Impuestos.php";
include "ImpuestosLocales.php";
include "LocalRetencion.php";
include "LocalTraslado.php";
include "Receptor.php";
include "RegimenFiscal.php";
include "Retencion.php";
include "Traslado.php";
include "ZipLibrary.php";

    
class CFDI33 {

        public $impuestos_conceptos;
        public $comprobante;
        public $emisor;
        public $receptor;
        public $conceptos;
        public $impuestos;
        public $impuestosLocales;
        public $complementos;
        public $complemento;
        public $addenda;
        private $xmlstream;
        private $cOriginal;
        private $cOriginalTFD;
        private $sign;
        private $csd;
        private $key;
        private $timbre;
        private $timeTimbre;
        private $noCertificadoTimbre;
        private $UUID;
        private $Cadena_original;

        function __construct(){
           $this->comprobante= new \Comprobante();
           $this->emisor= new \Emisor();
           $this->receptor = new \Receptor();
           $this->conceptos = new \Conceptos();
           $this->impuestos = new \Impuestos();
           $this->impuestosLocales = new \ImpuestosLocales();
           $this->ConceptoImpuestos = new \ConceptoImpuesto();
           $this->complementos = false;
           $this->impuestos_retencion = array();
           $this->impuestos_traslado = [];
           $this->addenda = [];
        }

         public function getNode(){
            $xml='<?xml version="1.0" encoding="UTF-8"?>
                  <cfdi:Comprobante  xmlns:cfdi="http://www.sat.gob.mx/cfd/3" 
                  xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
                  xsi:schemaLocation="http://www.sat.gob.mx/cfd/3 http://www.sat.gob.mx/sitio_internet/cfd/3/cfdv33.xsd" '.$this->comprobante->getAtributes().' >
                    '.$this->emisor->getNode().'
                    '.$this->receptor->getNode().'
                    '.$this->conceptos->getNode().'
                    '.$this->impuestos->getNode();
            if($this->complementos):
                    $xml.='<cfdi:Complemento>'.$this->impuestosLocales->getNode().'</cfdi:Complemento>';
            endif;
            $xml.='</cfdi:Comprobante>';
            $this->xmlstream = utf8_encode($xml);
            return $xml;
        }

        public function getCadenaOriginal($xmlstream=null){
          	$xml = new DomDocument();
            $original='';
            if($xmlstream==null):
	            $tmp=$this->xmlstream;
	            if($xml->loadXML($tmp)):
	                $xsl = new DomDocument;
                    if($xsl->load(__DIR__.'/xslt_sat/'.'cadenaoriginal_3_3.xslt')):
	                    $proc = new XsltProcessor();
	                    $proc->importStylesheet($xsl);
	                    $original =$proc->transformToXML($xml);
                    endif;
                endif;
            else:
                $tmp=$xmlstream;
                $tmp=str_ireplace('&amp;','@1@',$xmlstream);
                if($xml->loadXML($tmp)):
                    $xsl = new DomDocument;
                    $xsl->load(__DIR__.'/xslt_sat/'.'cadenaoriginal_3_3.xslt');
                    $proc = new XsltProcessor();
                    $proc->importStylesheet($xsl);
                    $original =$proc->transformToXML($xml);
                endif;
            endif;
            if(strlen($original)>0):
                $this->cOriginal=  $original;	
                return  $original;
            else:
                throw new Exception('No fue posible obtener la cadena original');
            endif;
        }

        public function getCadenaOriginalTFD($xmlstream=null){
            $xml = new DomDocument();
            $original='';
            if($xmlstream==null):
	            if($xml->loadXML($this->xmlstream)):
	                $xsl = new DomDocument;
	                if($xsl->load(__DIR__.'/xslt_sat/'.'cadenaoriginal_TFD_1_1.xslt')):
	                    $proc = new XsltProcessor();
	                    $proc->importStylesheet($xsl);
	                    $original =$proc->transformToXML($xml);
                    endif;
                endif;
            else:
 	            if($xml->loadXML($xmlstream)):
	                $xsl = new DomDocument;
	                $xsl->load($xmlstream);
	                $proc = new XsltProcessor();
	                $proc->importStylesheet($xsl);
	                $original =$proc->transformToXML($xml);
                endif;
            endif;
            if(strlen($original)>0):
                $this->cOriginalTFD= $original;
                $aux2 = explode('|',$this->cOriginalTFD);
                $this->UUID=$aux2[2];
                $this->timeTimbre=$aux2[3];
                $posI = strpos($this->xmlstream,'SelloSAT="')+strlen('SelloSAT="');
                $posF = strpos($this->xmlstream,'"',$posI+strlen('SelloSAT="'));
                $this->timbre=substr($this->xmlstream,$posI,$posF-$posI);
                $this->noCertificadoTimbre=$aux2[5];
                $this->cOriginalTFD='||'.$aux2[1].'|'.$aux2[2].'|'.$aux2[3].'|'.$aux2[4].'|'.$aux2[5].'||';
                return '||'.$aux2[1].'|'.$aux2[2].'|'.$aux2[3].'|'.$aux2[4].'|'.$aux2[5].'||';
            else:
                throw new Exception('No fue posible obtener la cadena original TFD');
            endif;
        }

        public function getUUID(){
          return $this->UUID;
        }

        public function getTimbreTime(){
          return $this->timeTimbre;
        }

        public function getTimbre(){
          return $this->timbre;
        }

        public function getCOriginalTFD(){
          return $this->cOriginalTFD;
        }

        public function getSello(){
          return $this->sign;
        }

        public function getnoCertificadoTimbre(){
          return $this->noCertificadoTimbre;
        }
    
        public function getQRCode($re='',$rr='',$Total='0.00'){
            $format = 'J';
            $size = '10';
            if($Total<0):
              $Total = $Total*-1;
            endif;
            $data = "?re=".$re."&rr=".$rr."&tt=".number_format($Total,6,'.','')."&id=".$this->UUID;
            qr_code($data, "M", $format, $size,7,$this->UUID);
        }

        public function getSign($coriginal=null){
          if($coriginal==null):
              $coriginal=utf8_encode($this->cOriginal);
              $coriginal=$this->cOriginal;
              $array=explode('>',$coriginal);
              $coriginal= trim($array[1]);
              $coriginal=str_replace('&amp;','&',$coriginal);
              $coriginal=str_replace('&quot;','"',$coriginal);
              $coriginal=str_replace('&apos;',"'",$coriginal);
              $coriginal=str_replace('&lt;',"<",$coriginal);
              $coriginal=str_replace('&gt;',">",$coriginal);
          endif;
          $fp = fopen(realpath($this->key), "r");
	      $priv_key = fread($fp, 8192);
	      fclose($fp);
	      $pkeyid = openssl_get_privatekey($priv_key);
	      openssl_sign($coriginal, $signature, $pkeyid, "SHA256");
	      openssl_free_key($pkeyid);
          $signature=base64_encode($signature);
          if(strlen($signature)>0):
               $this->sign =  $signature;
               return  $signature;
          else:
                throw new Exception('No fue posible obtener el sello');
          endif;
        }

        public function setCSD($csd){
            $this->csd=$csd;
        }

        public function setKEY($key){
            $this->key=$key;
        }

        private function getCSD(){
        	$fp = fopen(realpath($this->csd), "r");
	        $cert = fread($fp, 8192);
	        return base64_encode($cert);
        }

        public function getXML_InvoiceOne(){
            $this->getNode();
            $cadena = $this->getCadenaOriginal();
            $this->getSign($cadena);
            $this->xmlstream=str_replace('Sello=" "','Sello="'.$this->sign.'"',$this->xmlstream);
            $this->xmlstream=str_replace('Certificado=" "','Certificado="'.$this->getCSD().'"',$this->xmlstream);
            
        }
       
        public function getSHA1(){
          $array=explode('>',$this->cOriginal);
          return hash('sha256', trim($array[1]));
        }
    }
