<?php

namespace App\Helper;

use App\Denominations;
use App\PDF;
use App\Order;
use App\Helper\CodeHelper;
use App\Helper\ResponseHelper;
use App\Helper\S3Helper;
use App\Helper\UUIDHelper;
use App\Helper\FirebaseHelper;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Queue\SerializesModels;
use Illuminate\Queue\InteractsWithQueue;

class PDFHelper { //implements ShouldQueue {

    //use InteractsWithQueue, Queueable, SerializesModels;

    private $pdf;
    private $json;

    const WIDTH = 11.023622047;
    const HEIGHT = 16.929133858;
    const YI = 5.44;
    const XI = 2.81;
    const MIN_VAL = 12;
    const FMM = 0.196850394; //Representación de 5mm, base para mover elementos a posiciones customizadas

    public function __construct($json) {
        $this->json = $json;
    }

    public function handle(Order $order) {
        $denominations = Denominations::all();
        $urls=array();
        set_time_limit(0);
        foreach ($denominations as $denomination) {
            $this->setupDocuments();
            $quantity = 0;
            if (property_exists($this->json, $denomination->denomination)) {
                $quantity = $this->json->{$denomination->denomination};
            }
            if ($quantity == null || $quantity == 0) {
                continue;
            }
            if (($quantity < self::MIN_VAL && $quantity != 0) || $quantity%self::MIN_VAL != 0) {
                continue;
            }
            $pdf = PDF::getFoliogroup($quantity, $denomination->id, $order);
            $denomination1 = $denomination->denomination;
            $litrosx = $denomination->litrosx;
            $litrosy = $denomination->litrosy;
            $litrosnumerox = $denomination->litrosnumerox;
            $litrostextox = $denomination->litrostextox;
            $word = $denomination->word;
            $font = $denomination->font; 
            $fontx = $denomination->fontx;
            $i=0;
            for ($folio = $pdf->foliostart; $folio < $pdf->foliostart + $pdf->quantity; $folio+=12){        
                $pdf->status = 'in_process';
                $pdf->save();
                $i++;
                $this->addPage();
                $this->setPageNumber($i);
                $folios = $this->getFolios($folio);
                $this->setText($denomination1, $litrosx, $litrosy, $litrosnumerox, $litrostextox, $word, $font, $fontx, $folios);
                $uuids = UUIDHelper::getUUIDs(12, $folios, $pdf->id);
                $this->setQrCode(CodeHelper::generateQr($uuids));
                $this->setBarcode(CodeHelper::generateBarcode($uuids), $uuids);
            }
            $url = S3Helper::get()->save($this->pdf, $pdf);
            $pdf->url = $url;
            $pdf->status = 'created';
            $pdf->save();
            $urls[] = $url;
            FirebaseHelper::get()->commit();
        }
        $order->status = 'created';
        $order->save();
        return ResponseHelper::response('success', $urls);
    }

    public function generateTest($json) {
        $json = urldecode($json);
        $json = json_decode($json);
        $denominations = Denominations::all();
        $names = array();
        set_time_limit(0);
        foreach ($denominations as $denomination) {
            $this->setupDocuments();
            $quantity = $json->{$denomination->denomination};
            if ($quantity == null || $quantity == 0) {
                continue;
            }
            if (($quantity < self::MIN_VAL && $quantity != 0) || $quantity%self::MIN_VAL != 0) {
                return ResponseHelper::response('warning', "Se pidieron $quantity tarjetas de {$denomination->denomination}, deben ser ".self::MIN_VAL." o más y ser múltiplo de ".self::MIN_VAL);
            }
            $foliogroup = PDF::getFoliogroup($quantity, $denomination->id);
            $i=0;
            for ($folio = $foliogroup->foliostart; $folio < $foliogroup->foliostart + $foliogroup->quantity; $folio+=12){
                $i++;
                $this->addPage();
                $this->setPageNumber($i);
                $folios = $this->getFolios($folio);
                $this->setText($denomination->denomination, $denomination->litrosx, $denomination->litrosy, $denomination->litrosnumerox, $denomination->litrostextox, $denomination->word, $denomination->font, $denomination->fontx, $folios);
                $uuids = UUIDHelper::getUUIDs(12, $folios);
                $this->setQrCode(CodeHelper::generateQr($uuids));
                $this->setBarcode(CodeHelper::generateBarcode($uuids), $uuids);
            }
            $names[] = S3Helper::get()->save($this->pdf);
        }
        $this->pdf->Output($names[0], 'I');
    }

    private function setupDocuments() {
        $pageLayout = array(self::WIDTH, self::HEIGHT);
        $this->pdf = new \PDF('L', 'in', $pageLayout  , true, 'UTF-8', false);
        $this->pdf->setMargins(0, 0, 0, false);
        $this->pdf->setAutoPageBreak(TRUE, 0);
        $this->pdf->AddFont('Existence-Light','','Existence-Light.php');
    }

    private function addPage(){
        $this->pdf->AddPage();
        $this->pdf->SetTextColor(0);
    }

    private function setPageNumber($pageNumber) {
        //$x = 16.9
        $x = 14.67;
        $y = 0.18;
        $this->pdf->Rotate(0);
        $this->pdf->SetFont('Helvetica', '', 6);
        $this->pdf->SetTextColor(111,111,111);
        $this->pdf->SetXY($x,$y);
        //$this->pdf->Rotate(270, $x, $y);
        //$this->pdf->Text($x,$y,$pageNumber);
        $this->pdf->Cell(2,0,$pageNumber,0,1,'R');
        $this->pdf->SetTextColor(0);
    }

    private function getFolios($folio){
        return array(
            $folio + 5,
            $folio + 4,
            $folio + 3,
            $folio + 2,
            $folio + 1,
            $folio,
            $folio + 11,
            $folio + 10,
            $folio + 9,
            $folio + 8,
            $folio + 7,
            $folio + 6
        );
    }

    private function setText($denomination, $litrosx, $litrosy, $litrosnumerox, $litrostextox, $word, $font, $fontx, $folios) {
        $this->pdf->Rotate(0);
        $this->pdf->SetFont('Helvetica', 'B', 48);
        $y = 3.7;
        $dx = 2.27;
        for ($x=$dx; $x <= $dx + self::XI * 5; $x+=self::XI) { 
            $this->pdf->Rotate(270, $x, $y);
            $this->pdf->Text($x, $y, $denomination);
        }
        $y += self::YI;
        for ($x=$dx; $x <= $dx + self::XI * 5; $x+=self::XI) { 
            $this->pdf->Rotate(270, $x, $y);
            $this->pdf->Text($x, $y, $denomination);
        }   

        //Denominación en número (talón)
        $this->pdf->Rotate(0);
        $this->pdf->SetFont('Helvetica', 'B', 32);
        $y = 0.27;
        $dx = $litrosnumerox;
        $i = 0;
        for ($x=$dx; $x <= $dx + self::XI * 5; $x+=self::XI){
            $this->pdf->Text($x, $y, $denomination);
            $i++;
        }
        $y += self::YI;
        for ($x=$dx; $x <= $dx + self::XI * 5; $x+=self::XI){
            $this->pdf->Text($x, $y, $denomination);
            $i++;
        }

        //Palabra LITROS (talón)
        $this->pdf->Rotate(0);
        $this->pdf->SetFont('Helvetica', 'B', 6);
        $y = 0.72;
        $dx = $litrosx;
        for ($x=$dx; $x <= $dx + self::XI * 6; $x+=self::XI){
            if ($denomination == 1) {
                $this->pdf->Text($x, $y, "LITRO");
            }else {
                $this->pdf->Text($x, $y, "LITROS");
            }
        }
        $y += self::YI;
        for ($x=$dx; $x <= $dx + self::XI * 6; $x+=self::XI){
            if ($denomination == 1) {
                $this->pdf->Text($x, $y, "LITRO");
            }else {
                $this->pdf->Text($x, $y, "LITROS");
            }
        }

        //Palabra LITROS
        $this->pdf->Rotate(0);
        $this->pdf->SetFont('Helvetica', 'B', 18);
        $y = $litrosy;
        $dx = 1.85;
        for ($x=$dx; $x <= $dx + self::XI * 6; $x+=self::XI){
            $this->pdf->Rotate(270, $x, $y);
            if ($denomination == 1) {
                $this->pdf->Text($x, $y, "LITRO");
            }else {
                $this->pdf->Text($x, $y, "LITROS");
            }
        }
        $y += self::YI;
        for ($x=$dx; $x <= $dx + self::XI * 6; $x+=self::XI){
            $this->pdf->Rotate(270, $x, $y);
            if ($denomination == 1) {
                $this->pdf->Text($x, $y, "LITRO");
            }else {
                $this->pdf->Text($x, $y, "LITROS");
            }
        }

        //Denominación en texto
        $this->pdf->Rotate(0);
        $this->pdf->SetFont('Helvetica', 'B', $font);
        $y -= self::YI;
        $dx = $fontx;
        for ($x=$dx; $x <= $dx + self::XI * 5; $x+=self::XI){
            $this->pdf->Rotate(270, $x, $y);
            $this->pdf->Text($x, $y, '(' . $word . ')');
        }
        $y += self::YI;
        for ($x=$dx; $x <= $dx + self::XI * 5; $x+=self::XI){
            $this->pdf->Rotate(270, $x, $y);
            $this->pdf->Text($x, $y, '(' . $word . ')');
        }

        //Denominación en texto (talón)
        $this->pdf->Rotate(0);
        $this->pdf->SetFont('Helvetica', 'B', 6);
        $y = 0.8;
        $dx = $litrostextox;
        for ($x=$dx; $x <= $dx + self::XI * 6; $x+=self::XI){
            $this->pdf->Text($x, $y, '(' . $word . ')');
        }
        $y += self::YI;
        for ($x=$dx; $x <= $dx + self::XI * 6; $x+=self::XI){
            $this->pdf->Text($x, $y, '(' . $word . ')');
        }

        //Folios
        $this->pdf->Rotate(0);
        $this->pdf->SetFont('Helvetica', 'B', 9);
        $y = 1.037;
        $dx = 0.8;
        $i = 0;
        for ($x=$dx; $x <= $dx + self::XI * 5; $x+=self::XI){
            $this->pdf->Rotate(270, $x, $y);
            $this->pdf->Text($x, $y, PDF::make($folios[$i]));
            $i++;
        }
        $y += self::YI;
        for ($x=$dx; $x <= $dx + self::XI * 5; $x+=self::XI){
            $this->pdf->Rotate(270, $x, $y);
            $this->pdf->Text($x, $y, PDF::make($folios[$i]));
            $i++;
        }

        //Folios (talón)
        $this->pdf->Rotate(0);
        $this->pdf->SetFont('Helvetica', 'B', 9);
        $y = 0.34;
        $dx = 0.35;
        $i = 0;
        for ($x=$dx; $x <= $dx + self::XI * 5.1; $x+=self::XI){
            $this->pdf->Rotate(270, $x, $y);
            $this->pdf->Text($x, $y, PDF::make($folios[$i]));
            $i++;
        }
        $y += self::YI;
        for ($x=$dx; $x <= $dx + self::XI * 5.1; $x+=self::XI){
            $this->pdf->Rotate(270, $x, $y);
            $this->pdf->Text($x, $y, PDF::make($folios[$i]));
            $i++;
        }
    }

    private function setQrCode($qrs){
        $dx = 1.04;
        $y = 4.5;
        $h = 0.7;
        $i = 0;
        for ($x=$dx; $x <= $dx + self::XI * 6; $x+=self::XI){
            $this->pdf->Rotate(270, $x, $y);
            $this->pdf->ImageSVG($qrs[$i],$x,$y,$h);
            $i++;
        }
        $y+=self::YI;
        for ($x=$dx; $x <= $dx + self::XI * 6; $x+=self::XI){
            $this->pdf->Rotate(270, $x, $y);
            $this->pdf->ImageSVG($qrs[$i],$x,$y,null,$h);
            $i++;
        }
    }

    private function setBarcode($barcodes, $uuids) {
        $dx = 0.83;
        $y = 1.7;
        $w = 2.8;
        $h = 0.106;
        $i = 0;
        $this->pdf->Rotate(0);
        $this->pdf->SetFont('Existence-Light', '', 4);
        for ($x=$dx; $x <= $dx + self::XI * 5; $x+=self::XI){
            $this->pdf->Rotate(270, $x, $y);
            $this->pdf->ImageSVG($barcodes[$i],$x,$y,$w,$h);
            $this->pdf->ImageSVG($barcodes[$i],$x,$y+$h-0.011,$w,$h);
            $this->pdf->Rotate(270, $x, $y);
            $uuid = UUIDHelper::separate($uuids[$i]);
            $this->pdf->Text($x+0.55,$y+0.22,$uuid);
            $i++;
        }
        $y+=self::YI;
        for ($x=$dx; $x <= $dx + self::XI * 5; $x+=self::XI){
            $this->pdf->Rotate(270, $x, $y);
            $this->pdf->ImageSVG($barcodes[$i],$x,$y,$w,$h);
            $this->pdf->ImageSVG($barcodes[$i],$x,$y+$h-0.011,$w,$h);
            $this->pdf->Rotate(270, $x, $y);
            $uuid = UUIDHelper::separate($uuids[$i]);
            $this->pdf->Text($x+0.55,$y+0.22,$uuid);
            $i++;
        }
    }
}
