<?php

namespace App\Http\Controllers;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use DB;


class CFDIController extends Controller
{
    private $CFDI_Comprobante;
    private $Receptor_Data;
    private $Emisor_Data;
    private $Total_Comprobante;
    private $Sucursal_Expedicion;
    
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->CFDI_Comprobante =  null;
        $this->Receptor_Data = null;
        $this->Emisor_Data =  null;
        $this->Total_Comprobante =  0;
        $this->Sucursal_Expedicion =  null;
    }

    /**
     * Create a new CFDI Document.
     *
     * @return response
     */
    public function Timbrar($xml = null){
        $this->XML_Stream = $xml;
    }

    /**
     * Create a new CFDI Document.
     *
     * @return response
     */
    private function InitDocument(){
        $this->CFDI_Comprobante =  new \App\Core\CFDI33();
    }

    /**
     * Set Comprobante Data.
     *
     * @return Comprobante
     */
    private function GetComprobanteData(){
        $this->Sucursal_Expedicion = DB::select('SELECT * FROM branches WHERE id = ? ', [$this->Emisor_Data[0]->id]);
        $this->CFDI_Comprobante->comprobante->setAtribute('Fecha', \Carbon::now()->utc);
        $this->CFDI_Comprobante->comprobante->setAtribute('Total', $this->Total_Comprobante);
        $this->CFDI_Comprobante->comprobante->setAtribute('SubTotal', $this->Total_Comprobante - ($this->Total_Comprobante * 0.16000));
        $this->CFDI_Comprobante->comprobante->setAtribute('TipoDeComprobante', 'I');
        $this->CFDI_Comprobante->comprobante->setAtribute('LugarExpedicion', $this->Sucursal_Expedicion[0]->address3);
        $this->CFDI_Comprobante->comprobante->setAtribute('MetodoPago', 'PUE');
        $this->CFDI_Comprobante->comprobante->setAtribute('FormaPago', '03');
        $this->CFDI_Comprobante->comprobante->setAtribute('Folio', '1');
        $this->CFDI_Comprobante->comprobante->setAtribute('NoCertificado', '3000000000');
        $this->CFDI_Comprobante->comprobante->setAtribute('Moneda', 'MXN');
    }

    /**
     * Set Emistor Data.
     *
     * @return response
     */
    private function GetEmisorData(){
        $this->Emisor_Data = DB::select('SELECT * FROM fiscals where id = ?', [1]);
        $this->CFDI_Comprobante->emisor->setAtribute('Rfc',  $this->Emisor_Data[0]->rfc);
        $this->CFDI_Comprobante->emisor->setAtribute('Nombre', $this->Emisor_Data[0]->razon_fiscal);
        $this->CFDI_Comprobante->emisor->setAtribute('RegimenFiscal', $this->Emisor_Data[0]->regimen_fiscal);
    }

     /**
     * Set Receptor Data.
     *
     * @return response
     */
    private function GetReceptorData(){
        $this->CFDI_Comprobante->receptor->setAtribute('Rfc', $this->Receptor_Data[0]->rfc);
        $this->CFDI_Comprobante->receptor->setAtribute('UsoCFDI', $this->Receptor_Data[0]->uso_cfdi);
        $this->CFDI_Comprobante->receptor->setAtribute('Nombre', $this->Receptor_Data[0]->razon_social);
    }

    /**
     * Set Conceptos Data.
     *
     * @return response
     */
    private function GetConceptos($type = null){
        if($type == 'non_detail'):
            $Concep = $this->CFDI_Comprobante->conceptos->addConcepto();
            $Concep->setAtribute('Unidad', 'Litros');
            $Concep->setAtribute('ClaveProdServ', '70141508');
            $Concep->setAtribute('ClaveUnidad','58');
            $Concep->setAtribute('NoIdentificacion', '003');
            $Concep->setAtribute('Cantidad', '100');
            $Concep->setAtribute('Descripcion', 'Litros en vales');
            $Concep->setAtribute('ValorUnitario', '1.00');
            $Concep->setAtribute('Importe', '100.00');
            $this->CFDI_Comprobante->ConceptoImpuestos->setAtribute('Base', '100.00');
            $this->CFDI_Comprobante->ConceptoImpuestos->setAtribute('Impuesto', '002');
            $this->CFDI_Comprobante->ConceptoImpuestos->setAtribute('TasaOCuota', '0.160000');
            $this->CFDI_Comprobante->ConceptoImpuestos->setAtribute('Importe', '16.00');
            $this->CFDI_Comprobante->ConceptoImpuestos->setAtribute('TipoFactor', 'Tasa');
        endif;
    }

    /**
     * Set General Taxes Data.
     *
     * @return response
     */
    private function GetTaxes(){
        $traslado = $this->CFDI_Comprobante->impuestos->addTraslado();
        $traslado->setAtribute('Importe' , '16.00');
        $traslado->setAtribute('Impuesto', '002');
        $traslado->setAtribute('TasaOCuota', '0.160000');
        $traslado->setAtribute('TipoFactor', 'Tasa');
        $this->CFDI_Comprobante->impuestos->setAtribute('TotalImpuestosTrasladados', '16.00');
    }

    /**
     * Set Certificates.
     *
     * @return response
     */
    private function SetCertificates(){
        $this->CFDI_Comprobante->setCSD(realpath("XMLFacturacionElectronica/csdandkey/".$this->CSDFiles[0]));
        $this->CFDI_Comprobante->setKEY(realpath("XMLFacturacionElectronica/csdandkey/".$this->CSDFiles[1]));
    }

    /**
     * Create a CFDI instance.
     *
     * @return response
     */
    public function createCfdi(Request $request){
        if($request->has('debtorno')):
            $this->Receptor_Data = DB::select('SELECT * FROM debtors where debtorno = ?', [$request->input('debtorno')]);
            $this->Total_Comprobante = $request->input('total');
            if(!empty($this->Receptor_Data)):
                try{
                $this->InitDocument();
                $this->GetEmisorData();
                $this->GetComprobanteData();
                $this->GetReceptorData();  
                $this->GetConceptos($request->input('type'));                  
                $this->GetTaxes();                  
                $this->SetCertificates();                  
                return response()->json([
                    'message' => 'Todo Correcto',
                 ], 200);       
                }catch(\Exception $error){
                   return response()->json([
                     'message' => 'Error al generar el documento: '.$error->getMessage()
                   ], 400); 
                }
            else:
                return response()->json([
                    'message' => 'No se encontraron los datos del Receptor'
                 ], 400);
            endif;            
        else:
                return response()->json([
                    'message' => 'Cliente vacio'
                ], 400);
        endif;
    }
}
