<?php

namespace App\Http\Controllers;

use App\Denominations;
use App\Order;
use App\Helper\PDFHelper;
use App\Helper\ResponseHelper;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use DB;

class ValesController extends Controller {

    private $coupon;
    private $ordenDetail;
    private $totalQty;
    private $pages;
    private $porCode;

    const MIN_VAL = 12;

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    /*public function __construct(Request $request){
        $this->totalQty=0;
        $value=explode("|",$request->input('liters'));
        $qty=explode("|",$request->input('quantity'));
        for($i=0;$i<count($value);$i++):
            $this->ordenDetail[$i]["podLiter"]=(int)$value[$i];
            $this->ordenDetail[$i]["podQuantity"]=(int)$qty[$i];
            $this->totalQty+=$qty[$i];
        endfor; 
        $this->pages=ceil($this->totalQty/12);
    }*/

    /**
    * Create a CFDI instance.
    *
    * @return response
    */
    /*public function createPDF(Request $request){
        if ($this->totalQty<10):
            $result["code"] =330;
            $result["description"] ='La orden de impresión debe ser de 1200 vales como mínimo ';
        else:
            DB::beginTransaction();
            //Insertamos la orden de impresión
            $insert = DB::insert('INSERT INTO `printorder` (`por_quantity`,`por_pages`) values (?,?)', [$this->totalQty,$this->pages]);
            if($insert):
                $rs = DB::select('SELECT MAX(`por_code`) as current FROM `printorder`;');
                $this->porCode = $rs[0]->current;
                for($i=0;$i<count($this->ordenDetail);$i++):
                    $insert = DB::insert('INSERT INTO `printorder_detail` (`pod_liter`,`pod_quantity`,`por_code`) values (?,?,?)', [$this->ordenDetail[$i]["podLiter"],$this->ordenDetail[$i]["podQuantity"],$this->porCode]);
                    if($insert):
                        $rs = DB::select('SELECT MAX(`pod_code`) as current FROM `printorder_detail`;');
                        $this->ordenDetail[$i]["podCode"] = $rs[0]->current;
                        for($j=0;$j<$this->ordenDetail[$i]["podQuantity"];$j++):
                            $uuid=$this->ordenDetail[$i]["podCode"]."-".$this->porCode."-".$j;
                            $insert = DB::insert('INSERT INTO `coupon` (`cou_uuid`,`cou_liter`,`pod_code`) values (?,?,?)', [$uuid,$this->ordenDetail[$i]["podLiter"],$this->ordenDetail[$i]["podCode"]]);
                            if($insert):
                                $result["code"] =200;
                                $result["description"] ='A imprimir';
                            else:
                                DB::rollBack();
                                $result["code"] =331;
                                $result["description"] ='No se pudieron almacenar los cupones';
                                break;
                            endif;   
                        endfor;
                    else:
                        DB::rollBack();
                        $result["code"] =331;
                        $result["description"] ='El detalle de la orden de impresión para '.$this->ordenDetail[$i]["poLiter"].' litros de '.$this->ordenDetail[$i]["podQuantity"].' vales, no pudo ser almacenada';
                        break;
                    endif;
                endfor;
                DB::commit();
                $result=$this->createFile();                
            else:
                $result["code"] =331;
                $result["description"] ='La orden de impresión no pudo ser almacenada';
            endif;          
        endif;   
        return response()->json([
            'message' => $result["description"]
        ], $result["code"]); 
    }*/

    /**
     * @author Erik Camacho
     * @param $request Objeto request (debe ser un json con almenos una de las claves 1,5,10,20 o 50)
     * @return Json para indicar el status de la orden
     * Genera una orden y manda la generación del pdf a una cola para ser procesado por un cronjob
     * 
     */
    public function order(Request $request) {
        $json = $request->all();
        $denominations = Denominations::all();
        foreach ($denominations as $denomination) {
            $quantity = 0;
            if (array_key_exists($denomination->denomination, $json)){
                $quantity = $json[$denomination->denomination];
            }
            if ($quantity == 0) {
                continue;
            }
            if (($quantity < self::MIN_VAL && $quantity != 0) || $quantity%self::MIN_VAL != 0) {
                return ResponseHelper::response('warning', "Se pidieron $quantity tarjetas de {$denomination->denomination}, deben ser ".self::MIN_VAL." o más y ser múltiplo de ".self::MIN_VAL."");
            }
        }

        $order = new Order();
        $order->status = 'unauthorized';
        $order->percentage = 0;
        $order->json = json_encode($json);
        $order->save();
        return ResponseHelper::response('generating', $order->id);
    }

    /**
     * @author Erik Camacho
     * @return Estatus del la orden
     * Crojob encargado de genera los PDF que esté en cola
     *
     */
    public function cronjob() {
	set_time_limit(0);
	ini_set('memory_limit', '900MB');
        DB::connection('mysql')->disableQueryLog();    
        if (Order::where('status', 'generating')->exists()) {
            return;
        }
        $order = Order::where('status', 'authorized')->first();
        if (!isset($order)) {
            return;
        }
        $order->status='generating';
        $order->save();
        $pdfHelper = new PDFHelper(json_decode($order->json));
        return $pdfHelper->handle($order);
    }

    public function generateTest($request) {
        return (new PDFHelper(null))->generateTest($request);
    }

    public function getOrders(){
        return response()->json([
            'data' => Order::all(),
            'message' => 'Mostrando pedidos'
        ], 200);
    }
}
