<?php

namespace App;

use Illuminate\Database\Eloquent\Model;

class PDF extends Model{
    protected $table = "ws_pdf";

    /**
     * @author Erik Camacho
     * @param $input Número de folio
     * @return int Folio formateado
     * Obtiene un folio en entero y retorna un string con un formato correcto de este folio (agrega ceros)
     *
     */
    public static function make($input) {
    	return str_pad($input, 8, "0", STR_PAD_LEFT);
    }


    /**
     * @author Erik Camacho
     * @param $quantity Cantidad de folios a utilizar
     * @param $denomination ID de la denominación que usará los folios
     * @param $order Modelo de la orden
     * Genera el grupo de folios a usar en un PDF y crea una instancia del modelo PDF con esa información
     *
     */
    public static function getFoliogroup($quantity, $denomination, $order) {
 		$last = self::all()->last();
        $pdf = new PDF();
        $pdf->quantity = $quantity;
        $pdf->id_denomination=$denomination;
        $pdf->id_order = $order->id;
        $pdf->status = "ready";
        if ($last == null) { //No hay datos
            $pdf->foliostart = 1;
        } else { //Ya hay datos
            $pdf->foliostart = $last->foliostart + $last->quantity;
        }
        $pdf->save();
        return $pdf;
    }

    public function uuids() {
        return $this->hasMany('App\UUID');
    }

    public function order() {
        return $this->belongsTo('App\Order');
    }

    public function denomination() {
        return $this->belongsTo('App\Denominations');
    }
}
